	page	,132
;*****************************************************************************
;
;  Sample SMART HOSTESS Loader Program
;  Copyright (C) 1986 Control Systems, Inc.  All rights reserved.
;
;  The information and design disclosed herein were originated by and are
;  the property of Control Systems, Inc.  This software may only be used
;  on or in conjunction with the Control Systems, Inc. SMART HOSTESS
;  board.
;
;*****************************************************************************

	.xlist
	include	dpram.equ
	.list

	extrn com_data:word
	extrn com_data_end:word

;----------------------------------------------------------------------------
	
wait_time	equ	36	; approximately 2 seconds
				; 2 times 18.2 ticks/second
tmr_int		equ	1ch	; timer interrupt type

;----------------------------------------------------------------------------

stack	segment	stack 'stack'
	dw	64 dup ('@@')
stack	ends

;----------------------------------------------------------------------------

data	segment	public 'data'

tmr_vector	dd	?	; previous timer vector

logo_msg	equ	$
	db	'Sample SMART HOSTESS Loader Program',0dh,0ah
	db	'Copyright (C) 1986 Control Systems, Inc. '
	db	'All rights reserved.',0dh,0ah,'$'

load_msg1	equ	$
	db	'Loading the SMART HOSTESS Control Program ...','$'
load_msg2	equ	$
	db	' load complete',0dh,0ah,'$'

pgm_name	db	'ctrlpgm.bin',0
file_handle	dw	?
load_buffer	db	256 dup (?)

timeout_err	equ	$
	db	'timeout error: '
	db	'SMART HOSTESS adapter failed to respond',0dh,0ah,'$'
open_err	equ	$
	db	'open error: ctrlpgm.bin',0dh,0ah,'$'
read_err	equ	$
	db	'read error: ctrlpgm.bin',0dh,0ah,'$'
close_err	equ	$
	db	'close error: ctrlpgm.bin',0dh,0ah,'$'

enable_msg	equ	$
	db	1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

data	ends

;----------------------------------------------------------------------------

code	segment	public 'code'
	assume	cs:code, ds:data

main	proc	far
	push	ds			; save DOS return
	xor	ax,ax
	push	ax

	mov	ax,data			; establish data segment
	mov	ds,ax

	call	set_tmr			; activate timer

	mov	dx,offset logo_msg	; display logo message
	mov	ah,9
	int	21h

	mov	dx,offset load_msg1	; display load initiation message
	mov	ah,9
	int	21h

	call	load_pgm		; load ctrlpgm, interrupt comm proc
	jc	main_20			; exit if errors ...

	mov	dx,offset load_msg2	; display load completion message
	mov	ah,9
	int	21h

	mov	ax,dpram_addr
	mov	es,ax
	mov	si,offset interact_flag	; ES:SI=> interaction flag
	mov	cs:timer,wait_time	; wait for relocation to complete
main_10:
	mov	ax,es:[si]		; check interact flag
	cmp	ax,55aah		; relocation complete ?
	jz	main_20			;   yes ... continue
	mov	ax,cs:timer		; get timer count
	or	ax,ax			; timeout ?
	jnz	main_10			;   no ... continue
	mov	dx,offset timeout_err	; indicate error
	mov	ah,9
	int	21h
main_20:
	call	dpm_init		; init dual ported memory map
	call	enbl_adpt		; enable the adapter
	call	reset_tmr		; de-activate timer
	ret
main	endp

;----------------------------------------------------------------------------
;  name: load_pgm
;  function: upload SMART HOSTESS program
;
;  enter: 'pgm_name' = program file name,
;         'io_addr' (equate) = base I/O address of the SMART HOSTESS,
;         'dpram_addr' (equate) = base memory address of dual-ported RAM
;  exit: carry clear if successful, else carry set
;
;  registers AX, BX, CX, DX, SI, DI and ES altered

load_pgm	proc
; open control program file
	mov	dx,offset pgm_name	; ES:DX=> ASCIIZ path name
	xor	al,al			; access code = read only
	mov	ah,3dh			; function = open a file
	int	21h	
	jnc	load_pgm_02		; continue if no errors ...
	pushf				; save status
	mov	dx,offset open_err	; DS:DX=> error message
	mov	ah,9			; function = print string
	int	21h
	popf				; recover status	
	jmp	short load_pgm_99	; exit ...
; discard 1st 256 bytes of file
load_pgm_02:
	mov	file_handle,ax		; save the file handle
	mov	bx,ax			; BX = file handle
	mov	dx,offset load_buffer	; DS:DX=> buffer address
	mov	cx,256			; CX = number of bytes to read
	mov	ah,3fh			; function = read from device
	int	21h
	jnc	load_pgm_10		; continue if no errors ...
	pushf				; save status
	mov	dx,offset read_err	; DS:DX=> error message
	mov	ah,9			; function = print string
	int	21h
	popf				; recover status
	jmp	short load_pgm_99	; exit ...

; read control program file
load_pgm_10:
	mov	ax,dpram_addr
	mov	es,ax
	mov	di,offset user_area	; ES:DI=> dual_ported RAM
	mov	bx,file_handle		; BX = file handle
	mov	dx,offset load_buffer	; DS:DX=> buffer address
load_pgm_20:
	mov	cx,256			; CX = number of bytes to read
	mov	ah,3fh			; function = read from device
	int	21h
	jnc	load_pgm_30		; continue if no errors ...
	pushf				; save status
	mov	dx,offset read_err	; DS:DX=> error message
	mov	ah,9			; function = print string
	int	21h
	popf				; recover status
	jmp	short load_pgm_99	; exit ...
load_pgm_30:
	mov	si,offset load_buffer	; DS:SI=> load buffer
	mov	cx,ax			; setup number of bytes read
	rep	movsb			; move program to dual-ported RAM
	cmp	ax,256			; end of file ?
	jz	load_pgm_20		; no ... continue
; close control program file
	mov	ah,3eh			; function = close a file
	int	21h
	jnc	load_pgm_40		; exit if no errors ...
	pushf				; save status
	mov	dx,offset close_err	; DS:DX=> error message
	mov	ah,9			; function = print string
	int	21h
	popf				; recover status
	jmp	load_pgm_99		; exit ...
load_pgm_40:
	mov	di,offset interact_flag	; ES:DI=> interaction flag
	mov	es:[di],0aa55h		; indicate load complete
	mov	dx,io_addr+2		; interrupt comm processor
	out	dx,al
	clc				; indicate no errors
load_pgm_99:
	ret
load_pgm	endp

;----------------------------------------------------------------------------
;  name:	dpm_init
;  function:	set all variables in dual ported memory to initial values
;  enter:
;  exit:

dpm_init	proc
	mov	ax,dpram_addr
	mov	es,ax
	assume	es:nothing
	mov	di,offset user_area	; es:di = dual_ported RAM

	mov	ax,data
	mov	ds,ax
	assume	ds:data

	mov	ax,offset com_data	; start
	mov	cx,offset com_data_end	; end
	sub	cx,ax			; end - start = length

	mov	si,offset com_data	; ds:si = starting address
	rep	movsb			; move init stuff to dpm
	ret
dpm_init	endp

;----------------------------------------------------------------------------
;  name: enbl_adpt
;  function: enable the adapter to start looping for work to do

enbl_adpt	proc
	mov	ax,dpram_addr
	mov	es,ax			; es = dpram address
	assume	es:nothing

	mov	si,offset Comq		; start of comm proc queue
	mov	ax,es:[si].msgq_head	; ax = current msg number
	mov	cx,msg_len		; length of a message
	mul	cl			; head * 16 = offset
	lea	di,[si].msgq_area	; di = start of message area
	add	di,ax			; es:di = where to put message

	push	si			; save si
	mov	si,offset enable_msg	; ds:si = message to write
	mov	cx,msg_len		; length of message
	rep	movsb			; write the message

	pop	si			; restore si
	mov	ax,es:[si].msgq_head	; old head ptr
	inc	ax			; added one msg
	mov	es:[si].msgq_head,ax	; update head ptr

	mov	dx,io_addr+2		; interrupt comm proc
	out	dx,al

	ret
enbl_adpt	endp

;----------------------------------------------------------------------------
;  name: set_tmr
;  function: set user timer interrupt
;
;  enter: no parameters required
;  exit: no paramters returned
;
;  registers AX, BX and ES altered

set_tmr	proc
	xor	ax,ax
	mov	es,ax
	mov	bx,tmr_int*4		; ES:BX=> timer interrupt vector

	mov	ax,es:[bx]		; save current vector
	mov	word ptr tmr_vector,ax
	mov	ax,es:[bx+2]
	mov	word ptr tmr_vector+2,ax

	cli				; disable interrupts
	mov	ax,offset tmr_isr	; setup new vector
	mov	es:[bx],ax
	mov	ax,cs
	mov	es:[bx+2],ax
	sti				; re-enable interrupts
	ret
set_tmr	endp

;----------------------------------------------------------------------------
;  name: reset_tmr
;  function: reset user timer interrupt
;
;  enter: no parameters required
;  exit: no paramters returned
;
;  registers AX, BX and ES altered

reset_tmr	proc
	xor	ax,ax
	mov	es,ax
	mov	bx,tmr_int*4		; ES:BX=> timer interrupt vector

	cli				; disable interrupts
	mov	ax,word ptr tmr_vector	; restore previous vector
	mov	es:[bx],ax
	mov	ax,word ptr tmr_vector+2
	mov	es:[bx+2],ax
	sti				; re-enable interrupts
	ret
reset_tmr	endp

timer	dw	?			; timer count

;----------------------------------------------------------------------------
;  name: tmr_isr
;  function: service user timer interrupt
;
;  enter: from BIOS timer interrupt service routine
;  exit: to BIOS timer interrupt service routine

tmr_isr	proc
	cmp	cs:timer,0		; is timer zero ?
	jz	tmr_isr_99		;   yes ... exit
	dec	cs:timer		; decrement count
tmr_isr_99:
	iret
tmr_isr	endp

code	ends
	end	main
