;----------------------------------------------------------------------------
;	File: SHLIB.ASM
;	Description: a library of routines for the SMART HOSTESS board
;	Author: Control Systems, Inc.
;	Date: 12/18/87
;----------------------------------------------------------------------------

	.xlist
	include	dpram.equ
	.list

;----------------------------------------------------------------------------

_text	segment	byte public 'code'
_text 	ends	

_data	segment word public 'data'
_data	ends

const	segment	word public 'const'
const	ends

_bss	segment	word public 'bss'
_bss	ends

dgroup	group	const, _bss, _data
	assume	cs: _text, ds: dgroup, ss: dgroup, es: dgroup

;----------------------------------------------------------------------------

_data	segment

shopenmsg	db	13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
shclosmsg	db	14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

_data	ends

;----------------------------------------------------------------------------

_text segment

;----------------------------------------------------------------------------
;	Function: shopen(portnum)
;	Parameters: int portnum;	/* number of port to open (0-7) */
;	Return: 1 if successful, 0 if failed
;	Description: open a comm port on the SMART HOSTESS

	public	_shopen
_shopen		proc	near
	push	bp			; prolog
	mov	bp,sp

	mov	ax,dpram_addr
	mov	es,ax			; es = dpram address
	assume	es:nothing

	mov	bx,offset Comq		; start of comm proc queue
	mov	ax,es:[bx].msgq_head	; ax = current msg number
	inc	ax			; bump pointer
	and	ax,es:[bx].msgq_mask	; mask it
	cmp	ax,es:[bx].msgq_tail	; is msg area full?
	mov	ax,0			; assume it is
	jz	_shopen_10		; it is full

	push	si
	push	di
	push	ds
	mov	ax,_data
	mov	ds,ax
	assume	ds:_data

	mov	ax,es:[bx].msgq_head	; get head again
	mov	cx,msg_len		; length of a message
	mul	cl			; get message offset
	lea	di,[bx].msgq_area	; get destination
	add	di,ax			; es:di = destination
	mov	si,offset shopenmsg	; ds:si = source
	mov	ax,[bp+4]		; port number to open
	mov	[si+1],ax		; 2nd byte of msg is port number
	rep	movsb			; move the message

	mov	ax,es:[bx].msgq_head	; get head again
	inc	ax			; increment it
	and	ax,es:[bx].msgq_mask	; mask it
	mov	es:[bx].msgq_head,ax	; store new head

	mov	dx,io_addr+2		; interrupt comm proc
	out	dx,al
	pop	ds
	pop	di
	pop	si
	mov	ax,1			; retval = 1

_shopen_10:
	pop	bp			; epilog
	ret
_shopen		endp

;----------------------------------------------------------------------------
;	Function: shclose(portnum)
;	Parameters: int portnum;	/* number of port to close (0-7)
;	Returns: 1 if successful, 0 if failed
;	Description: close a comm port on the SMART HOSTESS

	public	_shclose
_shclose	proc	near
	push	bp			; prolog
	mov	bp,sp

	mov	ax,dpram_addr
	mov	es,ax			; es = dpram address
	assume	es:nothing

	mov	bx,offset Comq		; start of comm proc queue
	mov	ax,es:[bx].msgq_head	; ax = current msg number
	inc	ax			; bump pointer
	and	ax,es:[bx].msgq_mask	; mask it
	cmp	ax,es:[bx].msgq_tail	; is msg area full?
	mov	ax,0			; assume it is
	jz	_shclose_10		; it is full

	push	si
	push	di
	push	ds
	mov	ax,_data
	mov	ds,ax
	assume	ds:_data

	mov	ax,es:[bx].msgq_head	; get head again
	mov	cx,msg_len		; length of a message
	mul	cl			; get message offset
	lea	di,[bx].msgq_area	; get destination
	add	di,ax			; es:di = destination
	mov	si,offset shclosmsg	; ds:si = source
	mov	ax,[bp+4]		; port number to close
	mov	[si+1],ax		; 2nd byte of msg is port number
	rep	movsb			; move the message

	mov	ax,es:[bx].msgq_head	; get head again
	inc	ax			; increment it
	and	ax,es:[bx].msgq_mask	; mask it
	mov	es:[bx].msgq_head,ax	; store new head

	mov	dx,io_addr+2		; interrupt comm proc
	out	dx,al
	pop	ds
	pop	di
	pop	si
	mov	ax,1			; retval = 1
	
_shclose_10:
	pop	bp			; epilog
	ret
_shclose	endp

;----------------------------------------------------------------------------
;	Function: shread(buffer,cnt,portnum)
;	Parameters: char *buffer;	/* points to storage buffer */
;		    int cnt;		/* maximum count to read */
;		    int portnum;	/* number (0-7) of comm port */
;	Returns: integer number of bytes read
;	Description: read bytes from an open comm port on the SMART HOSTESS

	public	_shread
_shread		proc	near
	push	bp			; prolog
	mov	bp,sp
	sub	sp,2
	push	si
	push	di
	push	ds

	mov	ax,[bp+8]		; ax = port number
	call	shget_line		; si = line table
	mov	di,si			; di = line table
	mov	[bp-2],si		; also save it

	mov	ax,ds
	mov	es,ax			; es = data segment
	assume	es:nothing
	mov	ax,dpram_addr
	mov	ds,ax			; ds = dpm segment
	assume	ds:nothing

	mov	ax,[bp+6]		; ax = count
	mov	cx,ds:[si].Rxq_head	; get head
	mov	bx,ds:[di].Rxq_tail	; get tail
	sub	cx,bx			; cx = head - tail
	jb	_shread_10		; if cx < 0 jump
	call	 shmin			; get min of cx and ax
	mov	dx,cx			; store min in dx
	jcxz	_shread_30		; exit if min = 0 (i.e. empty)

	mov	si,ds:[di].Rxq_offset	; si = receive buffer
	add	si,bx			; si = current loc in buffer
	mov	di,[bp+4]		; di = local buffer
	rep	movsb			; read bytes into buffer
	jmp	short _shread_20	; jump
_shread_10:
	add	cx,ds:[di].Rxq_mask	; adjust cx
	inc	cx
	call	shmin			; now get min of cx and ax again
	mov	dx,cx			; store min
	jcxz	_shread_30		; exit if min = 0

	mov	ax,ds:[di].Rxq_mask	; ax = mask
	sub	ax,bx			; ax = mask - tail
	inc	ax			; ax = buffsize - tail
	mov	cx,ax			; cx = count I can read here
	mov	si,ds:[di].Rxq_offset	; si = receive buffer
	add	si,bx			; si = current loc in buffer
	mov	di,[bp+4]		; di = local buffer
	rep	movsb			; read bytes into buffer
	mov	cx,dx			; cx = full min count
	sub	cx,ax			; subtract what we just read
	jcxz	_shread_20		; exit if 0

	mov	bx,[bp-2]		; restore start of line table
	mov	si,ds:[bx].Rxq_offset	; restore start of buffer
	rep	movsb			; read rest into local buffer

_shread_20:
	mov	di,[bp-2]		; di = line table
	mov	bx,ds:[di].Rxq_offset	; bx = receive buffer
	sub	si,bx			; get new tail
	and	si,ds:[di].Rxq_mask	; mask it
	mov	ds:[di].Rxq_tail,si	; store it

_shread_30:
	mov	ax,dx			; ax = number of bytes read
	pop	ds			; epilog
	pop	di
	pop	si
	mov	sp,bp
	pop	bp
	ret
_shread		endp

;----------------------------------------------------------------------------
;	Function: shwrite(buffer,cnt,portnum)
;	Parameters: char *buffer;	/* points to buffer to write */
;		    int cnt;		/* number of bytes to write */
;		    int portnum;	/* number (0-7) of comm port */
;	Returns: integer number of bytes written
;	Description: write bytes to an open comm port on the SMART HOSTESS

	public	_shwrite
_shwrite	proc	near
	push	bp			; prolog
	mov	bp,sp
	sub	sp,2
	push	si
	push	di
	push	es

	mov	ax,dpram_addr		; set up dual port memory seg
	mov	es,ax
	assume	es:nothing

	mov	ax,[bp+8]		; get port number
	call	shget_line		; si = line table
	mov	[bp-2],si		; save the pointer

	mov	cx,es:[si].Txq_tail	; get tail
	mov	bx,es:[si].Txq_head	; get head
	sub	cx,bx			; get difference
	jbe	_shwrite_10		; branch if tail <= head

	mov	ax,[bp+6]		; ax = count
	dec	cx			; adjust tail - head
	call	shmin			; return minimum of ax and cx
	mov	dx,cx			; save minimum in dx
	jcxz	_shwrite_30		; exit if min = 0 (i.e. full)

	mov	di,es:[si].Txq_offset	; get offset of buffer
	add	di,bx			; get starting loc in buffer
	mov	si,[bp+4]		; si = start of write buffer
	rep	movsb			; write buffer to dpm
	jmp	short _shwrite_20	; done

_shwrite_10:
	mov	ax,[bp+6]		; ax = count
	add	cx,es:[si].Txq_mask	; adjust tail - head
	call	shmin
	mov	dx,cx			; store minimum in dx
	jcxz	_shwrite_30		; exit if min = 0

	mov	ax,es:[si].Txq_mask	; ax = mask
	sub	ax,bx			; ax -= head
	inc	ax			; # we can write at end of buffer
	call	shmin
	mov	di,es:[si].Txq_offset	; get buffer offset
	add	di,bx			; get starting loc in buffer
	mov	si,[bp+4]		; si = start of write buffer
	rep	movsb			; write buffer to dpm

	mov	cx,dx			; cx = count
	sub	cx,ax			; subtract what we just wrote
	jcxz	_shwrite_20		; exit if done

	mov	bx,[bp-2]		; get line table pointer
	mov	di,es:[bx].Txq_offset	; reset buffer pointer to start
	rep	movsb			; write rest of buffer
_shwrite_20:
	mov	si,[bp-2]		; si = line table
	mov	bx,es:[si].Txq_offset	; get offset of buffer
	sub	di,bx			; get new head pointer
	and	di,es:[si].Txq_mask	; mask it
	mov	es:[si].Txq_head,di	; store new head

_shwrite_30:
	mov	ax,dx			; ax = bytes written
	pop	es
	pop	di			; epilog
	pop	si
	mov	sp,bp
	pop	bp
	ret
_shwrite	endp

;----------------------------------------------------------------------------

shget_line	proc
	mov	cl,line_entry_len	; cl = length of a line table
	mul	cl			; mult portnum by length
	add	ax,offset line0		; add start of line tables
	mov	si,ax			; return in si
	ret
shget_line	endp

;----------------------------------------------------------------------------

shmin	proc
	cmp	ax,cx			; compare the two values
	jnc	shmin_10			; branch if ax >= cx
	mov	cx,ax			; make cx = ax
	jmp	short shmin_99
shmin_10:
	mov	ax,cx			; make ax = cx
shmin_99:
	ret
shmin	endp

;----------------------------------------------------------------------------

_text	ends
	end
